// SNMPSET.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"

/*
  snmpSet.cpp 

  version 2.8

  Copyright (c) 1999
  Hewlett-Packard Company

  ATTENTION: USE OF THIS SOFTWARE IS SUBJECT TO THE FOLLOWING TERMS.
  Permission to use, copy, modify, distribute and/or sell this software
  and/or its documentation is hereby granted without fee. User agrees
  to display the above copyright notice and this license notice in all
  copies of the software and any documentation of the software. User
  agrees to assume all liability for the use of the software; Hewlett-Packard
  makes no representations about the suitability of this software for any
  purpose. It is provided "AS-IS" without warranty of any kind,either express
  or implied. User hereby grants a royalty-free license to any and all
  derivatives based upon this software code base.

  Peter E. Mellquist
*/
#include "../snmp/snmp_pp.h"
//#include <iostream.h>
#include <stdlib.h>


// detrmine the smi type and get a value from
// the user
int determine_vb( SmiUINT32 val, Vb &vb) {
   char buffer[255];

   printf("Value Type is ");
   switch (val) {

	  // octet string
      case sNMP_SYNTAX_OCTETS:
	  {
	     printf("Octet String\nValue ?");
		 scanf("%s",buffer);
		 OctetStr octetstr( buffer);
		 if ( octetstr.valid()) {
		    vb.set_value( octetstr);
			return TRUE;
         }
		 else {
		   printf("Invalid OctetStr\n");
		   return FALSE;
         }
      }
	  break;

	  // IP Address
	  case sNMP_SYNTAX_IPADDR:
	  {
	     printf("IP Address\nValue ?");
	     scanf("%s",buffer);
	     IpAddress ipaddress( buffer);
	     if ( ipaddress.valid()) {
		    vb.set_value( ipaddress);
		    return TRUE;
         }
	     else {
		    printf("Invalid IP Address\n");
		    return FALSE;
         }
	  }
	  break;

	  // Oid
	  case sNMP_SYNTAX_OID:
	  {
	     printf("Oid\nValue ?");
	     scanf("%s",buffer);
	     Oid oid( buffer);
	     if ( oid.valid()) {
		    vb.set_value( oid);
		    return TRUE;
         }
	     else {
		    printf("Invalid Oid\n");
		    return FALSE;
         }
      }
	  break;

	  // TimeTicks
	  case sNMP_SYNTAX_TIMETICKS:
	  {
	     printf("TimeTicks\nValue ?");
		 scanf("%s",buffer);
         unsigned long i;
		 i = atol( buffer);
		 TimeTicks timeticks( i);
		 if ( timeticks.valid()) {
		    vb.set_value( timeticks);
			return TRUE;
         }
		 else {
			printf("Invalid TimeTicks\n");
			return FALSE;
         }
      }
	  break;

	  // Gauge32
      case sNMP_SYNTAX_GAUGE32:
	  {
         printf("Gauge32\nValue ?");
         scanf("%s",buffer);
         unsigned long i;
         i = atol( buffer);
         Gauge32 gauge32(i); 
         if ( gauge32.valid()) {
            vb.set_value( gauge32);
            return TRUE;
         }
         else {
            printf("Invalid Gauge32\n");
            return FALSE;
         }
      }
	  break;

	  case sNMP_SYNTAX_CNTR32:
      {
         printf("Counter32\nValue ?");
         scanf("%s",buffer);
         unsigned long i;
         i = atol( buffer);
         Counter32 counter32(i);
         if ( counter32.valid()) {
            vb.set_value( counter32);
            return TRUE;
         }
         else {
            printf("Invalid Counter32\n");
            return FALSE;
         }
      }
	  break;

	  case sNMP_SYNTAX_INT:
      {
         printf("Integer\nValue ?");
         scanf("%s",buffer);
         unsigned long i;
         i = atol( buffer);
         long l ;
		 l = ( long) i;
         vb.set_value( l);
         return TRUE;
      }
	  break;

	  case sNMP_SYNTAX_UINT32:
      {
         printf("Integer\nValue ?");
         scanf("%s",buffer); // Yes, I could use %d here, but its a port from cout
         unsigned long i;
         i = atol( buffer);
         vb.set_value( i);
         return TRUE;
      }
	  break;

	  default:
		 printf("Unknown Data Type\n");
		 return FALSE;
   }
}


int main( int argc, char **argv)  {

   //---------[ check the arg count ]----------------------------------------
   if ( argc < 2) {
	  printf("Usage:\n");
	  printf("snmpSet Address | DNSName [Oid] [options]\n");
	  printf("Oid: sysDescr object is default\n");
	  printf("options: -v1 , use SNMPV1, default\n");
	  printf("         -v2 , use SNMPV2\n");
	  printf("         -cCommunity_name, specify community default is 'public' \n");
	  printf("         -rN , retries default is N = 1 retry\n");
	  printf("         -tN , timeout in hundredths-seconds default is N = 100 = 1 second\n");
	  return 0;
   }

   //---------[ make a GenAddress and Oid object to retrieve ]---------------
   GenAddress address( argv[1]);      // make a SNMP++ Generic address
   if ( !address.valid()) {           // check validity of address
	  printf("Invalid Address or DNS Name, %s\n",argv[1]);
	  return 0;
   }
   Oid oid("1.3.6.1.2.1.1.4.0");      // defualt is sysName
   if ( argc >= 3) {                  // if 3 args, then use the callers Oid
	  if ( strstr( argv[2],"-")==0) {
	     oid = argv[2];
	     if ( !oid.valid()) {         // check validity of user oid
		    printf("Invalid Oid, %s\n",argv[2]);
		    return 0;
         }
      }
   }

   //---------[ determine options to use ]-----------------------------------
   snmp_version version=version1;                       // default is v1
   int retries=1;                                       // default retries is 1
   int timeout=100;                                     // default is 1 second
   OctetStr community("public");                        // community name
   char *ptr;
   for(int x=1;x<argc;x++) {                           // parse for version
      if ( strstr( argv[x],"-v2")!= 0)   
         version = version2c;
      if ( strstr( argv[x],"-r")!= 0) {                 // parse for retries
         ptr = argv[x]; ptr++; ptr++;
		 retries = atoi(ptr);
		 if (( retries<1)|| (retries>5)) retries=1; 
      }
	  if ( strstr( argv[x], "-t")!=0) {                 // parse for timeout
		 ptr = argv[x]; ptr++; ptr++; 
		 timeout = atoi( ptr);
		 if (( timeout < 100)||( timeout>500)) timeout=100;
      }
	  if ( strstr( argv[x],"-c")!=0) {
		 ptr = argv[x]; ptr++; ptr++;
		 community = ptr;
      }
   }

   //----------[ create a SNMP++ session ]-----------------------------------
   int status; 
   Snmp snmp( status);                // check construction status
   if ( status != SNMP_CLASS_SUCCESS) {
      printf("SNMP++ Session Create Fail, %s\n",snmp.error_msg(status));
      return 0;
   }

   //--------[ build up SNMP++ object needed ]-------------------------------
   Pdu pdu;                                // construct a Pdu object
   Vb vb;                                  // construct a Vb object
   vb.set_oid( oid);                       // set the Oid portion of the Vb
   pdu += vb;                              // add the vb to the Pdu
   CTarget target( address);               // make a target using the address
   target.set_version( version);           // set the SNMP version SNMPV1 or V2
   target.set_retry( retries);             // set the number of auto retries
   target.set_timeout( timeout);           // set timeout
   target.set_readcommunity( community);   // set read community
 
   //-------[ issue the request, blocked mode ]-----------------------------
   printf("SNMP++ Set to %s SNMPV%d Retries=%d",argv[1],(version+1),retries);

   printf(" Timeout=%dms\n",timeout);
   // first get the variabel to determine its type
   if (( status = snmp.get( pdu,target))== SNMP_CLASS_SUCCESS) {
	  pdu.get_vb( vb,0);
	  printf("Oid = %s\n",vb.get_printable_oid());
	  printf("Current Value = %s\n",vb.get_printable_value());
	  if ( determine_vb(vb.get_syntax(), vb)) {
	      // do the Set
		  Pdu setpdu;
		  vb.set_oid( oid);           // use the same oid as the inquire
		  setpdu += vb; 
	          status = snmp.set( setpdu, target);
		  printf("Set Status = ");
		  if ( status == SNMP_CLASS_ERR_STATUS_SET)
			  status = setpdu.get_error_status();
		  printf("%s\n",snmp.error_msg( status));
	  }
   }
   else {
	  if ( status == SNMP_CLASS_ERR_STATUS_SET)
		  status = pdu.get_error_status();
	  printf("SNMP++ Set Error, %s\n",snmp.error_msg( status));
   }
   return 0;
}  // end snmpSet

